//////////////////////////////////////////////////////////////////////////////
//   ///////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
#include "stdafx.h"
#include "_DRAW_2D.h"
#include "_MATH_Noise.h"


//////////////////////////////////////////////////////////////////////////////
int keyExit = 0, preset = 0, HelpTimer = 0;

//////////////////////////////////////////////////////////////////////////////////////////
inline tfloat sq(tfloat t) { return t * t; }; 
inline tfloat dist(tfloat x1, tfloat y1, tfloat x2, tfloat y2) { return sqrt(sq(x2 - x1) + sq(y2 - y1)); };

tfloat offs = 0.5, lim = 0.75, mult = 0.1;
Noise2D ns; 

inline tfloat force(tfloat r, tfloat x1, tfloat y1, tfloat x2, tfloat y2) 
{ 
  float t, d = dist(x1, y1, x2, y2);
  if (d > r) return 0;
  if ((t = (1 + sin(d * mult - offs)) * 0.5 * ns.GetNs(x1,y1) *  HExp(d, r)) > lim) return 1.0;
  else if (t > lim - 0.11) return 0.175; else return 0.0;
};

//////////////////////////////////////////////////////////////////////////////
#define count 40000

tfloat speed = 0.35, radius = 1.75, reaction = 0.3, falloff = 0.975, border = 1, scale = 1;

//////////////////////////////////////////////////////////////////////////////
void presetChange()
{
  preset = (preset + 1) % 2;
  switch(preset)
  { case 0: lim = 0.17; mult = 0.015; break;
    case 1: lim = 0.17; mult = 0.025; break;
  };
};

void keyPressed()                           //  [ ] 
{
  if (II.KeyUp[_Space_]) presetChange();
  if (II.KeyUp[_F1_]) if (HelpTimer) HelpTimer = 0; else HelpTimer = 25 * 10;
  
  keyExit = (II.KeyUp[_Esc_]);
  II.Reset();
};

//////////////////////////////////////////////////////////////////////////////
class c_circ
{
  public:
  tfloat rad, bord, x, y, dx, dy, xx, yy, ddx, ddy;
  void reset()
  {
     rad  = fRandom(radius * 0.35, radius * 1.15) * scale; 
     bord = rad * border;
     xx = x = fRandom(bord, DrawXL - bord);
     yy = y = fRandom(bord, DrawYL - bord);
     tfloat d = fRandom(3, 5) * speed * scale;
     ddx = 0; dx  = d * RandomSign();
     ddy = 0; dy  = d * RandomSign();
  };
  void regenerate()
  { 
    x += dx; y += dy;
    if ((x > (DrawXL - bord)) || (x < bord)) dx = - dx; else if (Random(37) == 11) dx = - dx;
    if ((y > (DrawYL - bord)) || (y < bord)) dy = - dy; else if (Random(37) == 11) dy = - dy;  
    tfloat d = dist(x, y, xx, yy);
    ddx += (x - xx) / d * reaction;
    ddy += (y - yy) / d * reaction;
    ddx *= falloff; ddy *= falloff;
    xx += ddx; yy += ddy;
  };
  void draw(int i, float r, dwRGB c, unsigned m)
  {
    DrawCircle(xx, yy, r * 5, RGBprocent(c, 25), m);
    DrawCircle(xx, yy, r, c, m);
  };
  void cdraw(int i, tfloat px1, tfloat py1, tfloat px2, tfloat py2, tfloat px3, tfloat py3, tfloat r)
  {
    r *= scale;
    dwRGB cc = 0;
    tfloat f1 = force(r, xx, yy, px1, py1),
           f2 = force(r, xx, yy, px2, py2),
           f3 = force(r, xx, yy, px3, py3),
           f = f1 + f2 + f3;
    if (f > 0.001)
    { cc = RGBadd(cc, RGBfmultV(RGBis(255, 100, 0), f1));
      cc = RGBadd(cc, RGBfmultV(RGBis(100, 225, 0), f2));
      cc = RGBadd(cc, RGBfmultV(RGBis(0, 100, 225), f3));
      draw(i, rad, cc, modeADD);
    };
  }; 
};

/////////////////////////////////////////////////////////////////////////////////////////////////
TextSPR *info;
c_circ circ[count],pcirc[3];

/////////////////////////////////////////////////////////////////////////////////////////////////
void CommonsInit()
{ 
  FC_FpsSet(AUTO);//    
  FC_DrawPaint = Yes;          //  fps   DrawPaint();
  RandomSeed();
  Draw2D_Init(); ForceActivate(); ScreenLoadWait();
  info = new TextSPR(L"TYPE_EN_RU");
  OnKeyUpDo(keyPressed);       //     
  keyPressed();                //     ...

  scale = (tfloat)DrawXR / 1600; 
  presetChange();
  ns.Make(10, 10, 50 * scale); 
    
  for (int i = 0; i < count; i++) circ[i].reset();
  
  speed = 2; radius = 150; 
  for (int i = 0; i < 3; i++) pcirc[i].reset();/**/
};


//////////////////////////////////////////////////////////////////////////////////////////////////
void CommonsDone()
{
  delete info;
  Draw2D_Done();
};

//////////////////////////////////////////////////////////////////////////////////////////////////
int infoDrawed = no;
void DrawInfo()
{
  if (!HelpTimer) return;
  HelpTimer--;
  infoDrawed = Yes;
  tRegion r = RegionCenterIs(200, 130);
  DrawRegionStore.Save(r);
  tfRegion fr = RegionTofRegion(r);
  fDrawRoundRect(fr, 20, RGBis(50, 50, 50), modeMUL); 
  info->Place(0, 0, modeADD + fromCC, "Conrol Keys:\n\n\nSpase - Change Preset\n...............................................................................\n\nF1 - hide/show Help\n\nEsc - Exit");
  info->Draw();
};

//////////////////////////////////////////////////////////////////////////////////////////////////
void RestoreInfo()
{
  if (!infoDrawed) return;
  DrawRegionStore.Restore();
  infoDrawed = no;
};

//////////////////////////////////////////////////////////////////////////////////////////////////
void DrawAll()
{
  //DrawFill(RGBis(125, 125, 125), modeMUL);
  DrawClear();
  offs += 0.1; if (offs > 2 * M_PI) offs -= 2 * M_PI; 
  for (int i = 0; i < 3; i++) pcirc[i].regenerate(); 
  for (int i = 0; i < count; i++)
  { circ[i].regenerate();
    circ[i].cdraw(i, pcirc[0].xx, pcirc[0].yy, pcirc[1].xx, pcirc[1].yy, pcirc[2].xx, pcirc[2].yy, 350);
  };
  DrawInfo(); 
  DrawPaint();
  RestoreInfo();
};

//////////////////////////////////////////////////////////////////////////////
//   //////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
void Main()
{
  CommonsInit();                  //   ;  
  
  while (!keyExit) DrawAll();

  CommonsDone();                  //   ;
};
//////////////////////////////////////////////////////////////////////////////
int LetProcess()
{
  //MouseShowCursor = no;
  WinName = L"CG : SMotion";   //    
  VS_CColor = RGBis(0, 0, 0);    //   
  DrawSet(FullScreen);         //   ( )  , FullScreen   
  ProcessFunc = Main;           //    -  
  return on;
};
int Init_LetProcess = LetProcess();

